-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 35 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=35;

--
-- Remove the non-null constraint for the service id associated with a config
-- revision since a revision can be tied to either a service or a host.
-- Create a new column in CONFIG_REVISIONS to store host id
--
alter table CONFIG_REVISIONS
      modify (SERVICE_ID null);

alter table CONFIG_REVISIONS
      add HOST_ID number;

--
-- Remove the non-null constraint for the service id associated with a config
-- since a config can be tied to either a service or a host.
-- Create a new column in CONFIGS to store host id
--
alter table CONFIGS
      modify (SERVICE_ID null);

alter table CONFIGS
      add HOST_ID number;

--
-- Each host with configs must be tied to a configuration revision. When the
-- configs are modified, a new revision is created and all non-inherited
-- configs are added to it.
-- Create a new column in HOSTS to refer to CONFIG_REVISIONS
--
alter table HOSTS
      add CONFIG_REVISION_ID number;

-- Insert a base revision for existing hosts.
-- Makes sure that the REVISION_IDs used for hosts are greater than
-- any existing REVISION_IDs in the CONFIG_REVISIONS table.
-- The value used for the CREATED_INSTANT is the UTC based epoch time, it
-- is equivalent to MySql 'unix_timestamp()'
-- 86400 = 24 * 60 * 60, number of seconds in a day
insert into CONFIG_REVISIONS (REVISION_ID, HOST_ID, CREATED_INSTANT, MESSAGE)
      select LAST_REV.MAX_ID + rownum REVISION_ID,
             h.HOST_ID,
             (cast(sys_extract_utc(systimestamp) as date) - date'1970-01-01') * 86400 CREATED_INSTANT,
             'Base revision' MESSAGE
      from HOSTS h,
           (select nvl(max(REVISION_ID), 0) MAX_ID from CONFIG_REVISIONS) LAST_REV;

--
-- Link each host to its current config revision.
--
update HOSTS h
      set CONFIG_REVISION_ID=
            (select REVISION_ID
	    from CONFIG_REVISIONS r
	    where h.HOST_ID=r.HOST_ID);

--
-- All data insertion is done. Set up indices and constraints.
--
create index IDX_CONFIG_HOST on CONFIGS (HOST_ID);

alter table CONFIGS
      add constraint FK_CONFIG_HOST
      foreign key (HOST_ID)
      references HOSTS
      deferrable initially immediate;

create index IDX_CONFIG_REVISION_HOST on CONFIG_REVISIONS (HOST_ID);

alter table CONFIG_REVISIONS
      add constraint FK_CONFIG_REVISION_HOST
      foreign key (HOST_ID)
      references HOSTS
      deferrable initially immediate;

create index IDX_HOST_CONFIG_REVISION on HOSTS (CONFIG_REVISION_ID);

alter table HOSTS
      add constraint FK_HOST_CONFIG_REVISION
      foreign key (CONFIG_REVISION_ID)
      references CONFIG_REVISIONS
      deferrable initially immediate;
